;
;       input.asm
;       TI-82 version
;
; Displays an "Eingabefeld"
;
; Last modified: 07/15/2001 mmddyyyy 15.39 by Burkart
;

#include "CRASH82.INC"

Str_ProgramTitle:
        .db "Eingabefeld Demo",0

ProgramStart:
        ROM_CALL(CLEARLCD)                      ; clear LCD
        ld      hl, $0000
        ld      (CURSOR_POS), hl
        ld      hl, Str_ProgramTitle            ; print Program title
        ROM_CALL(D_ZT_STR)

        ld      hl, $0006                       ; display key help
        ld      (CURSOR_POS), hl
        ld      hl, Str_KeyHelp
        ROM_CALL(D_ZT_STR)
        call    ShowDispMaxLen

        xor     a                               ; initialize StrLen
        ld      (Var_StrLen), a

        ld      hl, $0003                       ; move cursor
        ld      (CURSOR_POS), hl

Init:
        ld      a, $5F                          ; display cursor
        ROM_CALL(TX_CHARPUT)

        ld      a, (TiML_DispLen)               ; display blanks
        dec     a
        jr      z, Main_Loop
DisplayABlanks:
        ld      b, a
        ld      a, $E1
DisplayBlank:
        ROM_CALL(TX_CHARPUT) ; warning: TX_CHARPUT might convert some control chars
        djnz    DisplayBlank

Main_Loop:
        halt                                    ; wait for key while saving power
        call    GET_KEY                         ; get keypad data
        cp      G_GRAPH                         ; "GRAPH" -> exit
        ret     z
        cp      G_YEDIT                         ; "Y=" -> MaxLen++
        jr      z, IncMaxLen
        cp      G_WINDOW                        ; "WINDOW" -> MaxLen--
        jr      z, DecMaxLen
        cp      G_ZOOM                          ; "ZOOM" -> DispLen++
        jr      z, IncDispLen
        cp      G_TRACE                         ; "TRACE" -> DispLen--
        jp      z, DecDispLen
        cp      G_DEL                           ; "DEL" -> StrLen--
        jp      z, Backspace
        cp      G_CLEAR                         ; "CLEAR" -> clear string
        jp      z, ClearString

        cp      $0A                             ; is $0A < scancode < $30?
        jr      c, Main_Loop                     ; if not loop again
        cp      $30
        jr      nc, Main_Loop

        sub     $0A                             ; let hl point to the character of the pressed key
        ld      hl, Map_KeyToChar
        ld      b, $00
        ld      c, a
        add     hl, bc

        ld      a, (hl)                         ; was it a special key (i.e. ON, ENTER)?
        or      a
        jr      z, Main_Loop                     ; if so, loop again
        ; TI-ASCII ;-) code is now in a

        ld      d, a
        ld      a, (Var_StrLen)
        ld      hl, TiML_MaxLen
        cp      (hl)
        jr      z, Main_Loop
        inc     a
        ld      (Var_StrLen), a
        ld      c, a
        ld      b, 0
        ld      hl, Var_String-1
        add     hl, bc
        ld      (hl), d
        jr      DisplayString

IncMaxLen:
        ld      a, (TiML_MaxLen)
        inc     a
        jr      z, Main_Loop
        ld      (TiML_MaxLen), a
        cp      2
        jr      nz, IncMaxLen_Not2
        ; MaxLen=2 => DispLen=2
        ld      (TiML_DispLen), a
IncMaxLen_Not2:
        cp      3
        jp      nz, ProgramStart
        ; MaxLen=3 => DispLen>=3
        ld      (TiML_DispLen), a
        jp      ProgramStart

DecMaxLen:
        ld      a, (TiML_MaxLen)
        dec     a
        jr      z, Main_Loop
        ld      (TiML_MaxLen), a
        ld      hl, TiML_DispLen
        cp      (hl)
        jr      c, DecMaxLen_DispLenBigger
        jp      ProgramStart
DecMaxLen_DispLenBigger:
        ld      (TiML_DispLen), a
        jp      ProgramStart

IncDispLen:
        ld      a, (TiML_DispLen)
        ld      hl, TiML_MaxLen
        cp      (hl)
        jp      z, Main_Loop
        cp      16
        jp      z, Main_Loop
        inc     a
        ld      (TiML_DispLen), a
        jp      ProgramStart

DecDispLen:
        ld      a, (TiML_DispLen)
        cp      4
        jp      c, Main_Loop
        dec     a
        ld      (TiML_DispLen), a
        jp      ProgramStart

Backspace:
        ld      a, (Var_StrLen)
        or      a
        jp      z, Main_Loop ; already StrLen=0
        dec     a
        ld      (Var_StrLen), a
        jr      DisplayString

ClearString:
        xor     a
        ld      (Var_StrLen), a
;       jr      DisplayString

DisplayString:
        ld      hl, $0003                       ; set cursor position
        ld      (CURSOR_POS), hl

        ld      a, (Var_StrLen)
;       or      a                               ; Str_Len=0 -> re-initialize
;       jr      z, Init
        ld      hl, TiML_DispLen
        cp      (hl)
        jr      z, DispStr_eqDispLen
        jr      c, DispStr_smDispLen
        ; StrLen > DispLen
        ld      hl, TiML_MaxLen
        cp      (hl)
        jr      nz, DispStr_biDLneMaxLen
        ; StrLen > DispLen & StrLen = MaxLen (case IV)
        ld      hl, Var_String+1                ; goto char offset StrLen-DispLen+1
        ld      b, 0
        ld      c, a
        add     hl, bc
        ld      a, (TiML_DispLen)
        ld      c, a
        ld      d, a
        ld      a, l
        sub     c
        jr      nc, Over1
        dec     h
Over1:
        ld      l, a
        ld      a, d
        ld      de, Buf_DispStr                 ; copy to buffer
        dec     a                               ; copy DispLen-1 chars
        ld      c, a
        ldir                                    ; do it now!
        ld      hl, TiML_DispLen                ; finally display it
        ROM_CALL(D_LT_STR)
        jp      Main_Loop
DispStr_eqDispLen:
        ; StrLen = DispLen
        ld      hl, TiML_MaxLen
        cp      (hl)
        jr      z, DispStr_eqDLeqMaxLen
        ; StrLen = DispLen & StrLen < MaxLen (case III)
DispStr_biDLneMaxLen:
        ; StrLen > DispLen & StrLen < MaxLen (case III)
        ; a = (Var_StrLen)
        ld      hl, Var_String+2                ; goto char offset StrLen-DispLen+2
        ld      b, 0
        ld      c, a
        add     hl, bc
        ld      a, (TiML_DispLen)
        ld      c, a
        ld      d, a
        ld      a, l
        sub     c
        jr      nc, Over2
        dec     h
Over2:
        ld      l, a
        ld      a, d
        ld      de, Buf_DispStr                 ; copy to buffer
        sub     2                               ; copy DispLen-2 chars
        ld      c, a
        ldir                                    ; do it now!
        ld      a, $5F                          ; add cursor
        ld      (de), a
        ld      hl, TiML_DispLen                ; finally display it
        ROM_CALL(D_LT_STR)
        jp      Main_Loop
DispStr_eqDLeqMaxLen:
        ; StrLen = DispLen & StrLen = MaxLen (case II)
        ld      hl, Var_StrLen
        ROM_CALL(D_LT_STR)
        jp      Main_Loop
DispStr_smDispLen:
        ; StrLen < DispLen (case I)
        ld      hl, Var_StrLen                  ; display string
        push    af
        ROM_CALL(D_LT_STR)
        pop     af
        ld      b, a                            ; b = StrLen
        ld      a, $5F                          ; display cursor
        ROM_CALL(TX_CHARPUT)
        ld      a, (TiML_DispLen)               ; a = DispLen
        sub     b                               ; a = DispLen - StrLen
        dec     a                               ; a = DispLen - StrLen - 1
        or      a                               ; a = 0 => 0 blanks? -> Main Loop
        jp      z, Main_Loop
        jp      DisplayABlanks                  ; display blanks


ShowDispMaxLen:
        ld      hl, $0107                       ; move cursor
        ld      (CURSOR_POS), hl

        ld      hl, TiML_MaxLen                 ; display MaxLen
        call    DispHex

        ld      hl, $0907                       ; move cursor
        ld      (CURSOR_POS), hl

        ld      hl, TiML_DispLen                ; display DispLen
;       call    DispHex
;       ret

DispHex:
        ld      b, 2
        xor     a
DispHex_HexLoop:
        rld
        push    af
        add     a, '0'
        cp      '9'+1
        jr      c, DispHex_Number
        add     a, 'A'-'0'-10
DispHex_Number:
        ROM_CALL(TX_CHARPUT)
        pop     af
        djnz    DispHex_HexLoop
        rld
        ret


Var_StrLen = GRAPH_MEM   ; Byte
Var_String = GRAPH_MEM+1 ; 255 Bytes

TiML_MaxLen:
        .db 10
TiML_DispLen:
        .db 6
        .db $CF ; arrow to the left
Buf_DispStr:
        .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ; 15


Str_KeyHelp:
        .db "MaxLen DispLen  "
        .db "+  -    +  -  ",$DE,0
Map_KeyToChar:
        .db '"',"WRMH",0,0,"?0VQLG",0,0,".ZUPKFC",0," YTOJEB",0,0,"XSNIDA"
